//
//  BeautifyTool.swift
//  SuperVolley
//
//  Created by 张崇超 on 2018/5/4.
//  Copyright © 2018年 Zebra. All rights reserved.
//

import UIKit

class BeautifyTool: UIView {

    static let tool: BeautifyTool = BeautifyTool.loadXibView()
    @IBOutlet weak var showView: UIView!
    /// 点击事件
    private var clickAction: ((_ num: Int)->Void)?
    /// 单元格
    @IBOutlet weak var collectionView: UICollectionView!
    @IBOutlet weak var layout: UICollectionViewFlowLayout!
    private let dataList: [Int] = [0, 1, 2, 3, 4, 5]
    private let identifier: String = "BeautifyCell"
    
    /// 上一次点击的单元格
    static var lastClickIndex: Int = 0
    
    class func loadXibView() -> BeautifyTool {
        let tool = Bundle.main.loadNibNamed("BeautifyTool", owner: nil, options: nil)?.last as! BeautifyTool
        tool.frame = CGRect.init(x: 0.0, y: 0.0, width: kWidth, height: kHeight)
        tool.alpha = 0.0
        
        tool.showView.transform = CGAffineTransform.init(translationX: 0.0, y: kHeight)
        
        tool.collectionView.dataSource = tool
        tool.collectionView.delegate = tool
        tool.collectionView.contentInset = UIEdgeInsetsMake(0.0, 10.0, 0.0, 10.0)

        tool.layout.minimumLineSpacing = 0.0
        tool.layout.minimumInteritemSpacing = 0.0
        tool.layout.itemSize = CGSize.init(width: (kWidth - 20.0) / CGFloat(tool.dataList.count), height: 55.0)
        
        tool.collectionView.register(BeautifyCell.self, forCellWithReuseIdentifier: tool.identifier)
        
        tool.showView.corner(byRoundingCorners: [UIRectCorner.topLeft,  UIRectCorner.topRight], radii: 8.0)
        
        return tool
    }
    
    class func showBeautifyTool(clickAction: ((_ num: Int)->Void)?) {
        
        let tool = BeautifyTool.tool
        tool.clickAction = clickAction
        KkeyWindow?.addSubview(tool)
        tool.collectionView.reloadData()

        UIView.animate(withDuration: 0.3, animations: {
            
            tool.alpha = 1.0
            tool.showView.transform = CGAffineTransform.identity
            
        }) { (isOK) in
            
            
        }
    }
    
    override func touchesBegan(_ touches: Set<UITouch>, with event: UIEvent?) {
        
        BeautifyTool.tool.cancleAction()
        BeautifyTool.tool.clickAction?(BeautifyTool.lastClickIndex)
    }
    
    func cancleAction() {
        
        let tool = BeautifyTool.tool

        UIView.animate(withDuration: 0.3, animations: {
            
            tool.alpha = 0.0
            tool.showView.transform = CGAffineTransform.init(translationX: 0.0, y: kHeight)
            
        }) { (isOK) in
            
            tool.removeFromSuperview()
        }
    }
}

extension BeautifyTool: UICollectionViewDataSource, UICollectionViewDelegateFlowLayout {
    
    func collectionView(_ collectionView: UICollectionView, didSelectItemAt indexPath: IndexPath) {
        
        BeautifyTool.tool.cancleAction()
        BeautifyTool.tool.clickAction?(indexPath.row)
        BeautifyTool.lastClickIndex = indexPath.row
    }

    func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        
        return self.dataList.count
    }
    func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
        
        let cell = collectionView.dequeueReusableCell(withReuseIdentifier: self.identifier, for: indexPath) as! BeautifyCell
        
        if indexPath.row == BeautifyTool.lastClickIndex {
            
            cell.numL.textColor = UIColor.white
            cell.numL.backgroundColor = UIColor.red
            cell.numL.layer.borderColor = UIColor.red.cgColor
            
        } else {
            
            cell.numL.textColor = UIColor.darkGray
            cell.numL.backgroundColor = UIColor.white
            cell.numL.layer.borderColor = UIColor.lightGray.cgColor
        }
        cell.numL.text = "\(self.dataList[indexPath.row])"
        
        return cell
    }
}

extension UIView {
    
    func corner(byRoundingCorners corners: UIRectCorner, radii: CGFloat) {
        let maskPath = UIBezierPath(roundedRect: CGRect.init(x: 0.0, y: 0.0, width: kWidth, height: self.bounds.height), byRoundingCorners: corners, cornerRadii: CGSize(width: radii, height: radii))
        let maskLayer = CAShapeLayer()
        maskLayer.frame = CGRect.init(x: 0.0, y: 0.0, width: kWidth, height: self.bounds.height)
        maskLayer.path = maskPath.cgPath
        self.layer.mask = maskLayer
    }
}

class BeautifyCell: UICollectionViewCell {
    
    override init(frame: CGRect) {
        super.init(frame: frame)
        
        self.contentView.addSubview(self.numL)
    }
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    lazy var numL: UILabel = {
        let label = UILabel.init(frame: CGRect.init(x: 0.0, y: 0.0, width: 40.0, height: 40.0))
        label.center = self.contentView.center
        label.layer.cornerRadius = 20.0
        label.layer.borderWidth = 1.0
        label.layer.borderColor = UIColor.lightGray.cgColor
        label.clipsToBounds = true
        label.textAlignment = .center
        
        return label
    }()
}
