//
//  PlayerView.swift
//  SuperVolley
//
//  Created by 张崇超 on 2018/5/2.
//  Copyright © 2018年 Zebra. All rights reserved.
//

import UIKit
import PLPlayerKit

class PlayerView: UIView {

    static var instance: PlayerView!
    /// 撤销按钮
    @IBOutlet weak var cancleBtn: UIButton!
    /// 下一步按钮
    @IBOutlet weak var nextBtn: UIButton!
    /// 工具View
    @IBOutlet weak var bottomView: UIView!
    /// 播放地止
    private var videoPath: String = ""
    /// 取消事件
    private var cancleAction: (()->Void)?
    
    /// 视频播放器
    private var videoPlayer: PLPlayer!
    
    class func loadXibView(videoPath: String, canclaAction: (()->Void)?) -> PlayerView {
        
        let tool = Bundle.main.loadNibNamed("PlayerView", owner: nil, options: nil)?.last as! PlayerView
        tool.frame = CGRect.init(x: 0.0, y: 0.0, width: kWidth, height: kHeight)
        tool.videoPath = videoPath
        tool.cancleAction = canclaAction
        PlayerView.instance = tool
        tool.setupPlayer()

        // 开始播放
        tool.videoPlayer.play()
        PlayerView.instance.bringSubview(toFront: PlayerView.instance.bottomView)
        
        // 添加通知 进入前台
        NotificationCenter.default.addObserver(forName: NSNotification.Name.UIApplicationDidBecomeActive, object: nil, queue: OperationQueue.main, using: { (note) in
            
            tool.videoPlayer?.resume()
        })
        NotificationCenter.default.addObserver(forName: NSNotification.Name.UIApplicationDidEnterBackground, object: nil, queue: OperationQueue.main, using: { (note) in
            
            tool.videoPlayer?.pause()
        })
        
        // 动画执行
        tool.nextBtn.transform = CGAffineTransform.init(translationX: -kWidth / 2.0, y: 0.0)
        tool.cancleBtn.transform = CGAffineTransform.init(translationX: -kWidth / 2.0, y: 0.0)
        UIView.animate(withDuration: 0.3, animations: {
            
            tool.nextBtn.transform = CGAffineTransform.identity
            tool.cancleBtn.transform = CGAffineTransform.identity
            
        }) { (isOk) in
            
        }
        
        return tool
    }
    
    // 初始化播放器
    func setupPlayer() {
        
        let option = PLPlayerOption.default()
        let url = URL.init(fileURLWithPath: PlayerView.instance.videoPath)
        
        let player = PLPlayer.init(url: url, option: option)
        player?.delegate = PlayerView.instance
        player?.loopPlay = true
        player?.isBackgroundPlayEnable = false
        PlayerView.instance.addSubview(player!.playerView!)
        
        PlayerView.instance.videoPlayer = player
    }
    
    /// 返回事件
    @IBAction func backAction() {
        
        PlayerView.instance.cancleAction?()
        // 取消播放
        PlayerView.instance.videoPlayer = nil
        
        // 删除已存文件
        VideoDealTool.deleteVideo(path: VideoSavePath)
        
        UIView.animate(withDuration: 0.3, animations: {
            
            PlayerView.instance.alpha = 0.0
            
        }) { (isOK) in
            
            PlayerView.instance.removeFromSuperview()
            NotificationCenter.default.removeObserver(self)
        }
    }
    
    /// 下一步事件
    @IBAction func nextAction() {

        // 获取视频第一帧率
        //let image = self.getVideoThumbImg(videoUrl: URL.init(fileURLWithPath: PlayerView.instance.videoPath))
        
        
//        // 移除播放
//        PlayerView.instance.videoPlayer = nil
//        PlayerView.instance.removeFromSuperview()
//        // 移除通知
//        NotificationCenter.default.removeObserver(self)
    }
    
    /// 获取视频截图
    func getVideoThumbImg(videoUrl: URL) -> UIImage? {
        
        let avAsset: AVAsset = AVAsset.init(url: videoUrl)
        //生成视频截图
        let generator = AVAssetImageGenerator(asset: avAsset)
        generator.appliesPreferredTrackTransform = true
        
        let time = CMTimeMakeWithSeconds(Float64(1.0), 600)
        var actualTime: CMTime = CMTimeMake(0, 0)
        
        let imageRef: CGImage? = try? generator.copyCGImage(at: time, actualTime: &actualTime)
        
        var frameImg: UIImage?
        if let imageRef = imageRef {
            
            frameImg = UIImage.init(cgImage: imageRef)
        }
        return frameImg
    }
    
}

extension PlayerView: PLPlayerDelegate {
    
    func player(_ player: PLPlayer, statusDidChange state: PLPlayerStatus) {
        
        print("播放状态变更:\(state)")
    }
    
    func player(_ player: PLPlayer, stoppedWithError error: Error?) {
        
        self.showText("播放失败")
    }
    
}
