//
//  TakeVideoViewController.swift
//  SuperVolley
//
//  Created by 张崇超 on 2018/5/2.
//  Copyright © 2018年 Zebra. All rights reserved.
//

import UIKit
import Photos
import PLShortVideoKit

/// 闪退原因,未在七牛后台注册app,需要注册
/// 视频保存位置
let TakeVideoSavePath: String = NSHomeDirectory().appending("/Library/Caches/takeVideoTemp.mp4")

class TakeVideoViewController: UIViewController {
    
    /// 顶部View
    @IBOutlet weak var topView: UIView!
    /// 底部View
    @IBOutlet weak var bottomView: UIView!
    /// 录制View
    @IBOutlet weak var recordView: UIView!
    /// 旋转90度使用
    @IBOutlet weak var animationView: UIView!
    /// 红色图片
    @IBOutlet weak var redView: UIView!
    /// 下一步
    @IBOutlet weak var nextBtn: UIButton!
    /// 顶部约束 适配iOS10
    @IBOutlet weak var topSpaceCons: NSLayoutConstraint!
    
    /// 拍摄视频
    private var recorder: PLShortVideoRecorder!
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        
        self.recorder.startCaptureSession()
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.view.backgroundColor = UIColor.lightGray
        if Double.init(UIDevice.current.systemVersion)! < 11.0 {
            
            self.topSpaceCons.constant += 20.0
        }
        
        // 初始化摄像机
        self.setupCamera()
        // 预览图
        self.view.addSubview(self.recorder.previewView!)
        self.view.bringSubview(toFront: self.topView)
        self.view.bringSubview(toFront: self.bottomView)
            
        // 添加圆环
        self.animationView.layer.addSublayer(self.circleLayer)
        self.animationView.transform = CGAffineTransform.init(rotationAngle: -CGFloat.pi / 2.0)
        
        // 下一步按钮不可用
        self.nextBtn.backgroundColor = UIColor.lightGray
        self.nextBtn.isEnabled = false
       
        // 添加长按手势
        self.recordView.addGestureRecognizer(self.longPress)
        
        if !VideoDealTool.canTakeVideo() {
            
            self.navigationController?.popViewController(animated: true)
        }
    }
    
    // 初始化拍摄视频
    func setupCamera() {
        
        /// 视频配置
        let videoConfig: PLSVideoConfiguration = PLSVideoConfiguration.default()
        /// 音频配置
        let audioConfig: PLSAudioConfiguration = PLSAudioConfiguration.default()
        
        let recorder = PLShortVideoRecorder.init(videoConfiguration: videoConfig, audioConfiguration: audioConfig)
        recorder.outputFileType = PLSFileType.MPEG4
        recorder.maxDuration = 15.0
        recorder.minDuration = 1.0
        recorder.setBeautifyModeOn(true)
        recorder.setBeautify(0.0)
        recorder.setRedden(0.0)
        recorder.setWhiten(0.0)
        
        recorder.adaptationRecording = true 
        
        recorder.delegateQueue = DispatchQueue.main
        recorder.delegate = self
        
        self.recorder = recorder
    }
    
    /// 开始录制事件
    @objc func pressAction(tap: UILongPressGestureRecognizer) {
        
        let status = tap.state
        if status == UIGestureRecognizerState.began {
            
            // 开始录制
            print("开始长按")
            UIView.animate(withDuration: 0.3) {
                
                self.redView.transform = CGAffineTransform.init(scaleX: 0.5, y: 0.5)
                self.recordView.transform = CGAffineTransform.init(scaleX: 1.3, y: 1.3)
            }
            print("VideoSavePath:\(TakeVideoSavePath)")
            let fileUrl: URL = URL.init(fileURLWithPath: TakeVideoSavePath)
            // 文件是否存在, 移除之前的文件
            self.recorder.deleteAllFiles()
            VideoDealTool.deleteVideo(path: TakeVideoSavePath)
            self.recorder.startRecording(fileUrl)
            
        } else if status == UIGestureRecognizerState.ended {
            
            self.endRecord(isStopRecord: true)
        }
    }
    
    // 录制结束
    func endRecord(isStopRecord: Bool) {
        
        UIView.animate(withDuration: 0.3) {
            
            self.redView.transform = CGAffineTransform.identity
            self.recordView.transform = CGAffineTransform.identity
            self.nextBtn.backgroundColor = UIColor.red
        }
        // 隐藏进度
        self.circleLayer.isHidden = true
        self.circleLayer.strokeEnd = 0.0

        // 停止视频编码输出
        if isStopRecord {
            self.recorder.stopRecording()
        }
    }
    
    /// 下一步事件
    @IBAction func nextAction(_ sender: UIButton) {
        
    }
    
    /// 返回事件
    @IBAction func backAction() {
        
        self.recorder.delegate = nil
        self.recorder = nil
        
        self.navigationController?.popViewController(animated: true)
    }
    
    /// 滤镜事件
    @IBAction func libraryAction() {
        
        BeautifyTool.showBeautifyTool { (index) in
            
            DispatchQueue.main.async {
                
                let num: CGFloat = 0.2 * CGFloat(index)
                self.recorder.setBeautify(num)
                self.recorder.setRedden(num)
                self.recorder.setWhiten(num)
            }
        }
    }
    
    /// 闪光灯事件
    @IBAction func lightAction() {
        
        if self.recorder.captureDevicePosition == .front { return }
        self.recorder.isTorchOn = !self.recorder.isTorchOn
    }

    /// 翻转事件
    @IBAction func reverAction() {
        
        self.recorder.toggleCamera()
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
    }
        
    /// 添加圆环
    lazy var circleLayer: CAShapeLayer = {
        let circleLayer = CAShapeLayer.init()
        let circleRadius: CGFloat = 35.0
        
        circleLayer.frame = self.recordView.bounds
        circleLayer.lineWidth = 5.0
        circleLayer.fillColor = UIColor.clear.cgColor
        circleLayer.strokeColor = UIColor.red.cgColor
        circleLayer.backgroundColor = UIColor.clear.cgColor
        
        // 设置圆环路径
        var circleFrame = CGRect.init(x: 0.0, y: 0.0, width: 2.0 * circleRadius, height: 2.0 * circleRadius)
        circleFrame.origin.x = circleLayer.bounds.midX - circleFrame.midX
        circleFrame.origin.y = circleLayer.bounds.midY - circleFrame.midY
        
        let circlePath = UIBezierPath.init(ovalIn: circleFrame)
        circleLayer.path = circlePath.cgPath
        
        // 初始化圆圈
        circleLayer.strokeEnd = 0
        
        return circleLayer
    }()
    
    /// 长按手势
    lazy var longPress: UILongPressGestureRecognizer = {
        let press = UILongPressGestureRecognizer.init(target: self, action: #selector(pressAction))
        press.minimumPressDuration = 0.01
        return press
    }()
}

extension TakeVideoViewController: PLShortVideoRecorderDelegate {
    
    // 开始录制
    func shortVideoRecorder(_ recorder: PLShortVideoRecorder, didStartRecordingToOutputFileAt fileURL: URL) {
        
        self.showText("开始录制")
    }
    
    // 结束录制
    func shortVideoRecorder(_ recorder: PLShortVideoRecorder, didFinishRecordingToOutputFileAt fileURL: URL, fileDuration: CGFloat, totalDuration: CGFloat) {
        
        print("结束录制:\(fileDuration)")
        // 判断时间是否大约1秒
        if fileDuration < 1.0 {

            self.showText("录制时间不能少于1秒")
            return
        }
        // 按钮可用
        self.nextBtn.isEnabled = true
        
        // 停止视频采集
        self.recorder.stopCaptureSession()
        
        let tool = VideoClipTool.loadXibView(videoUrl: fileURL, viewController: self, dealVideoPath: { (path) in
            
            // 完成
        })
        let pushVC = UIViewController.init()
        pushVC.view = tool
        
        let nav = AppDelegate.cureentRoot
        nav?.pushViewController(pushVC, animated: true)
    }
 
    // 正在录制中
    func shortVideoRecorder(_ recorder: PLShortVideoRecorder, didRecordingToOutputFileAt fileURL: URL, fileDuration: CGFloat, totalDuration: CGFloat) {
        
        print("录制中:\(fileDuration)")
        self.circleLayer.isHidden = false
        self.circleLayer.strokeEnd += fileDuration / 2280.0
        if fileDuration > 15.0 {
            
            // 结束
            self.circleLayer.strokeEnd = 1.0
            self.endRecord(isStopRecord: false)
        }
    }
}
