//
//  VideoDealTool.swift
//  SuperVolley
//
//  Created by 张崇超 on 2018/5/9.
//  Copyright © 2018年 Zebra. All rights reserved.
//

import UIKit
import AVFoundation
import Photos

class VideoDealTool: NSObject {

    //MARK: -是否支持拍摄视频
    @discardableResult
    static func canTakeVideo() -> Bool {
        
        let audioStatus = AVCaptureDevice.authorizationStatus(for: AVMediaType.video)
        if audioStatus == .denied || audioStatus == .restricted {

//            AlertTool.show(title: "温馨提示", subTitle: "相机&麦克风服务已关闭，请到“设置>隐私>相机&麦克风”中开启【SuperVolley】相机&麦克风服务,获取您的相机&麦克风权限", leftText: "取消", rightText: "确定") {
//
//                let url = URL.init(string: UIApplicationOpenSettingsURLString)
//                UIApplication.shared.openURL(url!)
//            }

        } else if audioStatus == .notDetermined {
            
            return false
        }
        return true
    }
    
    /// 是否可以保存图片/视频到图库
    @discardableResult
    static func canSaveMediaToAlbum() -> PHAuthorizationStatus {
        
        let status = PHPhotoLibrary.authorizationStatus()
        if status == .restricted || status == .denied {
            
//            AlertTool.show(title: "无法保存照片/视频", subTitle: "请在iPhone的\"设置-隐私-照片\"中允许访问图库", leftText: "取消", rightText: "设置", rightAction: {
//
//                let url = URL.init(string: UIApplicationOpenSettingsURLString)
//                UIApplication.shared.openURL(url!)
//            })
        }
        return status
    }
    
    //MARK: -下载文件
    static func downLoadFile(path: String, hudView: UIView) {
        
        if VideoDealTool.canSaveMediaToAlbum() == .notDetermined {
            
            // 获取权限
            let firstPath = Bundle.main.path(forResource: "firstVideo", ofType: "mp4")
            VideoDealTool.saveMediaToLocal(img: nil, path: firstPath, result: nil)
            
            PHPhotoLibrary.requestAuthorization { (status) in
                
                DispatchQueue.main.async {
                    
                    self.downLoadFile(path: path, hudView: hudView)
                }
            }
            
        } else if VideoDealTool.canSaveMediaToAlbum() == .authorized {
            
            
        }
    }
    
    //MARK: -保存图片/视频到本地
    static func saveMediaToLocal(img: UIImage? = nil, path: String? = nil, result: ((_ result: Bool)->Void)?) {
        
        // 权限问题
        let status = VideoDealTool.canSaveMediaToAlbum()
        if status == .restricted || status == .denied {
            
            return
        }
        PHPhotoLibrary.shared().performChanges({
            
            if let img = img {
                
                PHAssetChangeRequest.creationRequestForAsset(from: img)
                
            } else if let path = path {
                
                let url: URL = URL.init(fileURLWithPath: path)
                PHAssetChangeRequest.creationRequestForAssetFromVideo(atFileURL: url)
            }
            
        }, completionHandler: { (isOk, error) in
            
            result?(isOk)
            if let error: NSError = error as NSError?, error.code == 2047 {
                
                PHPhotoLibrary.requestAuthorization { (status) in
                    
                    DispatchQueue.main.async {
                        
                        self.saveMediaToLocal(img: img, path: path, result: result)
                    }
                }
            }
        })
    }

    //MARK: -删除已存在的视频 否则视频录制无法存储
    static func deleteVideo(path: String) {
        
        let fileMgr = FileManager.default
        if fileMgr.fileExists(atPath: path) {
            
            do {
                
                try fileMgr.removeItem(atPath: path)
                
            } catch {
                
                // 删除失败, 重新保存
            }
        }
    }
    
    //MARK: -获取相册的视频地址
    static func getVideoUrlFromAsset(asset: PHAsset, result: ((_ url: URL?)->Void)?) {
        
        let options = PHVideoRequestOptions.init()
        options.version = PHVideoRequestOptionsVersion.current
        options.deliveryMode = .automatic
        
        PHImageManager.default().requestAVAsset(forVideo: asset, options: options) { (avasset, avaudio, dic) in
            
            if let avUrlAsset: AVURLAsset = avasset as? AVURLAsset {
                
                result?(avUrlAsset.url)
                
            } else {
                
                result?(nil)
            }
        }
    }
}
