//
//  MediaViewTool.swift
//  相册访问Demo
//
//  Created by 张崇超 on 2018/6/6.
//  Copyright © 2018年 ZCC. All rights reserved.
//
import UIKit
import Photos

let kWidth = UIScreen.main.bounds.size.width
let kHeight = UIScreen.main.bounds.size.height
let KkeyWindow = UIApplication.shared.keyWindow

class MediaViewTool: UIView {

    /// 选中的图片 PHAsset对象
    var lastChioceAssetsArr: [PHAsset]?
    /// 选中的图片 UIImage对象
    var chioceImgsArr: [UIImage] = []
    
    /// 顶部View
    @IBOutlet weak var topView: UIView!
    @IBOutlet weak var rightNavL: UILabel!
    /// 视频按钮
    @IBOutlet weak var videoBtn: UIButton!
    ////标记符
    @IBOutlet weak var identifierView: UIView!
    /// 集合视图
    @IBOutlet weak var collectionView: UICollectionView!
    @IBOutlet weak var layout: UICollectionViewFlowLayout!
    
    /// 单元格大小
    private let cellWH: CGFloat = (kWidth - 2.0) / 3.0
    /// 视频资源
    private var videosArr: [PHAsset] = []
    /// 图片资源
    private var imgsArr: [PHAsset] = []
    /// 所有资源
    private var allDataArr: [PHAsset] = []
    /// 需要展示的数据类型
    private var showDataArr: [PHAsset] = []
    /// 是否在执行动画
    private var isAnimating: Bool = false
    /// 传进来的控制器
    private var currentVC: UIViewController!
    /// 选择图片的回调
    private var pickImgsCallBack: ((_ imgsArr: [UIImage]) -> Void)?
    /// 选择视频的回调
    private var pickVideoCallBack: ((_ path: String) -> Void)?
    /// 记录上一次的偏移量
    private var imgContentOffset: CGPoint?
    private var videoContentOffset: CGPoint?
    
    static func showPhotoChioceTool(baseVC: UIViewController,
                                    imgCallBack: ((_ imgs: [UIImage]) -> Void)?,
                                    videoCallBack: ((_ videoPath: String) -> Void)?) {
        
        let tool = MediaViewTool.loadXibView()
        if tool.isAnimating { return }
        tool.isAnimating = true
        
        tool.alpha = 0.0
        tool.transform = CGAffineTransform.init(translationX: 0.0, y: kHeight)
        baseVC.view.addSubview(tool)
        tool.currentVC = baseVC
        tool.pickImgsCallBack = imgCallBack
        tool.pickVideoCallBack = videoCallBack
        
        UIView.animate(withDuration: 0.3, delay: 0.0, usingSpringWithDamping: 1.0, initialSpringVelocity: 0.0, options: .allowAnimatedContent, animations: {
            
            tool.transform = CGAffineTransform.identity
            tool.alpha = 1.0

        }) { (isOk) in
            
            tool.getAllPhotos()
            tool.isAnimating = false
            tool.btnChioceAction(tool.videoBtn)
        }
    }
    
    class func loadXibView() -> MediaViewTool {
        let tool = Bundle.main.loadNibNamed("MediaViewTool", owner: nil, options: nil)?.last as! MediaViewTool
        tool.frame = CGRect.init(x: 0.0, y: 0.0, width: kWidth, height: kHeight)
        
        return tool
    }
    
    override func awakeFromNib() {
        super.awakeFromNib()
        
        self.topView.corner(byRoundingCorners: [.topLeft, .topRight], radii: 4.0)
        self.layout.itemSize = CGSize.init(width: cellWH, height: cellWH)
        self.layout.minimumLineSpacing = 1.0
        self.layout.minimumInteritemSpacing = 1.0
        
        self.collectionView.register(MediaViewToolCell.self, forCellWithReuseIdentifier: "MediaViewToolCell")
        self.collectionView.delegate = self
        self.collectionView.dataSource = self
    }
    
    func getAllPhotos() {
        
        DispatchQueue.main.async {
            
            self.imgsArr.removeAll()
            self.videosArr.removeAll()
            self.allDataArr.removeAll()
            
            PHPhotoLibrary.shared().register(self)
            let allOptions = PHFetchOptions.init()
            allOptions.sortDescriptors = [NSSortDescriptor.init(key: "creationDate", ascending: false)]
            
            let allResult: PHFetchResult<PHAsset> = PHAsset.fetchAssets(with: allOptions)
            for index in 0..<allResult.count {
                
                let asset: PHAsset = allResult[index]
                if asset.mediaType == .image {
                    
                    self.imgsArr.append(asset)
                    
                } else if asset.mediaType == .video {
                    
                    self.videosArr.append(asset)
                }
                self.allDataArr.append(asset)
            }
            self.showDataArr = self.videosArr
            self.collectionView.reloadData()
        }
    }
    
    /// 导航栏右侧按钮点击事件
    @IBAction func rightnavAction() {
        
        self.cancleAction()
        if self.rightNavL.text!.contains("开拍") {
            
            let vc = TakeVideoViewController.init(nibName: "TakeVideoViewController", bundle: nil)
            AppDelegate.cureentRoot.pushViewController(vc, animated: true)
            
        } else {
            
            if self.chioceImgsArr.isEmpty {
                
                self.pickVideoCallBack?(VideoSavePath)
                
            } else {
                
                self.pickImgsCallBack?(self.chioceImgsArr)
            }
        }
    }
    
    /// 视频/图片 事件
    @IBAction func btnChioceAction(_ sender: UIButton) {
        
        DispatchQueue.main.async {
            
            if self.identifierView.center.x == sender.center.x { return }
            if self.isAnimating { return }
            self.isAnimating = true
            
            UIView.animate(withDuration: 0.3, delay: 0.0, usingSpringWithDamping: 0.6, initialSpringVelocity: 0.0, options: .allowAnimatedContent, animations: {
                
                var newCenter = self.identifierView.center
                newCenter.x = sender.center.x
                self.identifierView.center = newCenter
                
            }) { (isOk) in
                
                if sender.tag == 101 {
                    
                    self.lastChioceAssetsArr = nil
                    self.rightNavL.text = "开拍>"
                    
                    // 滚动到指定区域
                    self.collectionView.contentOffset = self.videoContentOffset ?? CGPoint.zero
                    
                } else {
                    
                    // 滚动到指定区域
                    self.collectionView.contentOffset = self.imgContentOffset ?? CGPoint.zero
                }
                sender.tag == 101 ? (self.showDataArr = self.videosArr) : (self.showDataArr = self.imgsArr)
                self.collectionView.reloadData()
                self.isAnimating = false
            }
        }
    }
    
    /// 取消事件
    @IBAction func cancleAction() {
        
        DispatchQueue.main.async {
            
            if self.isAnimating { return }
            self.isAnimating = true
            
            UIView.animate(withDuration: 0.3, animations: {
                
                self.alpha = 0.4
                self.transform = CGAffineTransform.init(translationX: 0.0, y: kHeight)
                
            }) { (isOk) in
                
                self.isAnimating = false
            }
        }
    }
}

extension MediaViewTool: UICollectionViewDataSource, UICollectionViewDelegateFlowLayout, PHPhotoLibraryChangeObserver {
  
    func collectionView(_ collectionView: UICollectionView, didSelectItemAt indexPath: IndexPath) {
        
        let asset = self.showDataArr[indexPath.row]
        if asset.mediaType == .image {

            let cell = collectionView.cellForItem(at: indexPath) as! MediaViewToolCell
            let originalFrame = collectionView.convert(cell.frame, to: self)
            CheckView.showAnimation(currentView: self, asset: asset, img: cell.imgV.image, originalFrame: originalFrame)
            
        } else {
            
            let options = PHVideoRequestOptions.init()
            options.version = PHVideoRequestOptionsVersion.current
            options.deliveryMode = .automatic
            
            self.showLoading("视频处理中")
            PHImageManager.default().requestAVAsset(forVideo: asset, options: options) { (avasset, avaudio, dic) in
                
                DispatchQueue.main.async {
                    
                    self.hideHUD()
                    if let avUrlAsset: AVURLAsset = avasset as? AVURLAsset {
                        
                        let tool = VideoClipTool.loadXibView(videoUrl: avUrlAsset.url, viewController: self.currentVC, dealVideoPath: { (path) in
                            
                            // 完成
                            self.cancleAction()
                            self.pickVideoCallBack?(VideoSavePath)
                        })
                        let pushVC = UIViewController.init()
                        pushVC.view = tool
                        let nav = AppDelegate.cureentRoot
                        nav?.pushViewController(pushVC, animated: true)
                        
                    } else {
                        
                        self.showText("iCould同步中,请稍后再试")
                    }
                }
            }
        }
    }
    
    func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        
        return self.showDataArr.count
    }
    
    func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
        
        let cell = collectionView.dequeueReusableCell(withReuseIdentifier: "MediaViewToolCell", for: indexPath) as! MediaViewToolCell
        
        cell.asset = self.showDataArr[indexPath.row]
        cell.topImgV.image = #imageLiteral(resourceName: "photo_preview_unselected")
        if let selectedArr = self.lastChioceAssetsArr, selectedArr.contains(cell.asset) {

            cell.topImgV.image = #imageLiteral(resourceName: "photo_preview_selected")
        }
        
        cell.chioceCallback = { asset, img, isSelected in
            
            if self.lastChioceAssetsArr == nil {
                
                self.lastChioceAssetsArr = []
            }
            if isSelected {
                
                self.chioceImgsArr.append(img)
                self.lastChioceAssetsArr?.append(asset)

            } else {
                
                let index = self.lastChioceAssetsArr?.index(of: asset)
                if let index = index {
                    
                    self.chioceImgsArr.remove(at: index)
                    self.lastChioceAssetsArr?.remove(at: index)
                }
            }
            var text: String = "开拍>"
            self.lastChioceAssetsArr!.isEmpty ? (text = "开拍>") : (text = "完成(\(self.lastChioceAssetsArr!.count))")
            self.rightNavL.text = text
        }
        return cell
    }
    
    func scrollViewDidScroll(_ scrollView: UIScrollView) {
        
        if self.identifierView.center.x == self.videoBtn.center.x {
            
            // 视频
            self.videoContentOffset = scrollView.contentOffset
            
        } else {
            
            // 图片
            self.imgContentOffset = scrollView.contentOffset
        }
    }
    
    // 第一次获取权限时使用
    func photoLibraryDidChange(_ changeInstance: PHChange) {
        
        self.getAllPhotos()
        self.btnChioceAction(self.videoBtn)
    }
}

class MediaViewToolCell: UICollectionViewCell {
    
    static var imgManager: PHImageManager = PHImageManager.default()
    
    /// 选中回调
    var chioceCallback: ((_ asset: PHAsset, _ img: UIImage ,_ isSelected: Bool)->Void)?
    
    var asset: PHAsset! {
        
        willSet {
            
            if newValue == nil { return }
            
            self.timeL.isHidden = !(newValue.mediaType == .video)
            self.timeL.setText(duration: newValue.duration)
            
            self.topImgV.isHidden = !(newValue.mediaType == .image)
            
            /// 模糊图 UIScreen.main.bounds.size
            let options = PHImageRequestOptions.init()
            options.resizeMode = .fast
            MediaViewToolCell.imgManager.requestImage(for: newValue, targetSize: CGSize.init(width: kWidth, height: kWidth), contentMode: PHImageContentMode.default, options: options) { (img, dic) in
                
                self.imgV.image = img
                if self.imgV.image == nil {
                    
                    self.imgV.image = #imageLiteral(resourceName: "imgError")
                }
            }
        }
    }
    
    override init(frame: CGRect) {
        super.init(frame: frame)
        
        self.contentView.addSubview(self.imgV)
        self.contentView.addSubview(self.timeL)
        self.contentView.addSubview(self.topView)
    }
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    /// 选中取消选中点击事件
    @objc func topImgVClickAction() {
        
        if self.topImgV.image == #imageLiteral(resourceName: "photo_preview_selected") {
            
            self.topImgV.image = #imageLiteral(resourceName: "photo_preview_unselected")
            
        } else {
            
            self.topImgV.image = #imageLiteral(resourceName: "photo_preview_selected")
        }
        self.topImgV.addAnimation(isSelected: self.topImgV.image == #imageLiteral(resourceName: "photo_preview_selected"))
        self.chioceCallback?(self.asset, self.imgV.image ?? UIImage() , self.topImgV.image == #imageLiteral(resourceName: "photo_preview_selected"))
    }
    
    /// 点击事件
    lazy var topImgVTap: UITapGestureRecognizer = {
        let tap = UITapGestureRecognizer.init(target: self, action: #selector(topImgVClickAction))
        return tap
    }()
    /// 左上角选择按钮
    lazy var topView: UIView = {
        let view = UIView.init(frame: CGRect.init(x: self.contentView.frame.width - 35.0, y: 0.0, width: 35.0, height: 35.0))
        view.addSubview(self.topImgV)
        view.addGestureRecognizer(self.topImgVTap)

        return view
    }()
    lazy var topImgV: UIImageView = {
        let imgWH: CGFloat = 20.0
        let imgV = UIImageView.init(frame: CGRect.init(x: 10.0, y: 5.0, width: imgWH, height: imgWH))
        imgV.contentMode = .scaleAspectFit
        imgV.image = #imageLiteral(resourceName: "photo_preview_unselected")
        
        return imgV
    }()
    
    /// 封面图
    lazy var imgV: UIImageView = {
        let imgV = UIImageView.init(frame: CGRect.init(x: 0.0, y: 0.0, width: self.contentView.frame.width, height: self.contentView.frame.height))
        imgV.contentMode = .scaleAspectFill
        imgV.clipsToBounds = true

        return imgV
    }()
    /// 视频时间
    lazy var timeL: UILabel = {
        let height: CGFloat = 18.0
        let label = UILabel.init(frame: CGRect.init(x: 0.0, y: self.contentView.frame.maxY - height, width: self.contentView.frame.width - 10.0, height: height))
        label.textAlignment = .right
        label.font = UIFont.systemFont(ofSize: 12.0)
        label.textColor = UIColor.white
        
        return label
    }()
}

class CheckView: UIView {
    
    static var selfInit: CheckView?
    /// 图片赋值
    var img: UIImage? {
        
        willSet {
            
            self.checkImgV.image = newValue
        }
    }
    /// 传递过来的动画原始位置
    private var originalFrame: CGRect = CGRect.zero
    
    // 初始化
    convenience init(img: UIImage? = nil) {
        self.init(frame: CGRect.init(x: 0.0, y: 0.0, width: kWidth, height: kHeight))
        
        self.clipsToBounds = true
        self.img = img
        self.addSubview(self.checkImgV)
        self.addGestureRecognizer(self.tap)
    }

    /// 展示图片
    class func showAnimation(currentView: UIView? = nil, asset: PHAsset, img: UIImage?, originalFrame: CGRect) {
        
        let tool = CheckView.init(img: nil)
        CheckView.selfInit = tool
        // 记录原始位置,用于缩小动画
        tool.originalFrame = originalFrame
        
        // 添加动画View
        tool.falseImgV.frame = originalFrame
        tool.falseImgV.alpha = 1.0
        tool.falseImgV.image = img
        (currentView ?? KkeyWindow)?.addSubview(tool.falseImgV)
        // 添加最终展示的View
        tool.img = img
        tool.alpha = 0.0
        (currentView ?? KkeyWindow)?.addSubview(tool)
        
        // 执行放大动画
        UIView.animate(withDuration: 0.2, animations: {
            
            tool.falseImgV.frame = CGRect.init(x: 0.0, y: 0.0, width: kWidth, height: kHeight)
            
        }) { (isOk) in
            
            // 隐藏动画View
            tool.falseImgV.alpha = 0.0
            
            // 加载高清图
            let options = PHImageRequestOptions.init()
            options.resizeMode = .fast
            MediaViewToolCell.imgManager.requestImage(for: asset, targetSize: PHImageManagerMaximumSize, contentMode: PHImageContentMode.default, options: options) { (img, dic) in
                
                if img != nil {
                    
                    tool.img = img
                }
            }
            // 展示出最终View
            tool.alpha = 1.0
        }
    }
    
    /// 展示完毕,移除
    func hiddenAnimation(originalFrame: CGRect) {
        
        let tool = CheckView.selfInit
        // 展示动画View
        tool?.falseImgV.alpha = 1.0
        // 执行缩小动画
        UIView.animate(withDuration: 0.2, animations: {
            
            tool?.falseImgV.frame = originalFrame
            tool?.falseImgV.contentMode = .scaleAspectFill
            tool?.falseImgV.clipsToBounds = true
            
        }) { (isOk) in
            
            UIView.animate(withDuration: 0.1, animations: {
                
                tool?.falseImgV.alpha = 0.0
                
            }, completion: { (isOk) in
                
                tool?.falseImgV.removeFromSuperview()
            })
        }
        // 移除视图
        CheckView.selfInit = nil
        tool?.removeFromSuperview()
    }
    
    //MARK: -点击事件
    /// 图片点击事件
    @objc func tapAction() {
        
        let tool = CheckView.selfInit
        tool?.hiddenAnimation(originalFrame: tool!.originalFrame)
    }
    //MARK: -懒加载
    /// 图片点击手势
    lazy var tap: UITapGestureRecognizer = {
        let tap = UITapGestureRecognizer.init(target: self, action: #selector(tapAction))
        return tap
    }()
    /// 最终展示的图
    lazy var checkImgV: UIImageView = {
        let imgV = UIImageView.init(frame: CGRect.init(x: 0.0, y: 0.0, width: self.frame.width, height: self.frame.height))
        imgV.contentMode = .scaleAspectFit
        imgV.backgroundColor = UIColor.black
        
        return imgV
    }()
    /// 假的View,用于处理放大动画
    lazy var falseImgV: UIImageView = {
        let imgV: UIImageView = UIImageView.init()
        imgV.contentMode = .scaleAspectFit
        imgV.backgroundColor = UIColor.black
        
        return imgV
    }()
}

extension UIImageView {
    
    func addAnimation(isSelected: Bool) {
        
        let animation = CAKeyframeAnimation(keyPath: "transform.scale")
        animation.duration = 0.5;
        animation.calculationMode = kCAAnimationCubic;
        
        if (isSelected) {
            
            animation.values = [1.5, 0.8, 1.0, 1.2, 1.0]
            
        } else {
            
            animation.values = [1.0, 0.6, 0.8, 1.0]
        }
        self.layer.add(animation, forKey: "transform.scale")
    }
}

extension UILabel {
    
    func setText(duration: Double) {

        if self.isHidden { return }
        
        var minute: Double = 0.0
        var second: Double = 0.0
        // 时间处理
        if duration / 60.0 > 0.0 {
            
            // 超过一分钟
            minute = duration / 60.0
            second = duration.truncatingRemainder(dividingBy: 60.0)
            
        } else if duration / 60.0 <= 0.0 {
            
            // 未超过一分钟
            minute = 0.0
            second = duration
        }
        let minuteStr = String.init(format: "%02ld", Int(minute))
        let secondStr = String.init(format: "%02ld", Int(second))
        self.text = minuteStr + ":" + secondStr
    }
}
