//
//  VideoClipTool.swift
//  VideoClipDemo
//
//  Created by 张崇超 on 2018/5/16.
//  Copyright © 2018年 zcc. All rights reserved.
//

import UIKit
import PLPlayerKit

/// 两个滑块的相距距离 一个单元格的宽度 - 右边父控件的空白宽度
let spaceMagin: CGFloat = kWidth / 7.0 - 10.0
/// 滑块的宽度
let sliderWidth: CGFloat = 10.0

/// 视频保存位置
let VideoSavePath: String = NSHomeDirectory().appending("/Library/Caches/shortVideoTemp.mp4")

class VideoClipTool: UIView {

    /// 播放区域
    @IBOutlet weak var playerView: UIView!
    /// 集合视图
    @IBOutlet weak var collectionView: UICollectionView!
    @IBOutlet weak var layout: UICollectionViewFlowLayout!
    /// 上下的线
    @IBOutlet weak var topView: UIView!
    @IBOutlet weak var bottomView: UIView!
    /// 左右View
    @IBOutlet weak var leftView: UIView!
    @IBOutlet weak var rightView: UIView!
    /// 左右两边间隔
    @IBOutlet weak var leftSpaceCons: NSLayoutConstraint!
    @IBOutlet weak var rightSpaceCons: NSLayoutConstraint!
    /// 位移动画使用
    @IBOutlet weak var animationView: UIView!
    /// 控制器
    private var viewController: UIViewController?
    /// 播放器
    private var player: PLPlayer?
    /// 视频地址
    private var videoUrl: URL?
    /// 视频截图
    private var dataList: [UIImage?] = []
    /// 单元格大小
    private let cellWidth: CGFloat = kWidth / 7.0
    private let cellheight: CGFloat = kWidth / 7.0 * 1.5
    /// 播放比例
    private var asecptNum: CGFloat = 0.0
    /// 左侧滑块的位移距离
    private var leftTime: CGFloat = 0.0
    /// 右侧滑块的位移距离
    private var rightTime: CGFloat = 15.0
    /// 集合视图的位移距离
    private var leftScroTime: CGFloat = 0.0
    /// 视频总时长
    private var totalTime: CGFloat = 0.0

    /// 开始时间
    private var beginTime: CGFloat = 0.0
    /// 结束时间
    private var endTime: CGFloat = 15.0
    /// 定时器
    private var timer: CADisplayLink?
    /// 视频地址回调
    private var dealVideoPathCallback: ((_ path: String)->Void)?
    /// 计算左右滑块距离
    private var spaceDistance: CGFloat = kWidth - (kWidth / 7.0) * 2.0
    /// 记录上一次播放位置 用于位移动画
    private var lastPlayTime: CGFloat?
    
    // 初始化
    class func loadXibView(videoUrl: URL, viewController: UIViewController? = nil, dealVideoPath: ((_ path: String)->Void)? ) -> VideoClipTool {
        let tool = Bundle.main.loadNibNamed("VideoClipTool", owner: nil, options: nil)?.last as! VideoClipTool
        tool.frame = CGRect.init(x: 0.0, y: 0.0, width: kWidth, height: kHeight)
        tool.viewController = viewController
        tool.videoUrl = videoUrl
        tool.dealVideoPathCallback = dealVideoPath
        
        tool.setupPlayer(videoUrl: videoUrl)
        
        tool.dataList = tool.getVideoThumbImg(videoUrl: videoUrl)
        
        // 设置位置
        if tool.dataList.count < 5 {
            
            // 视频时长不足15秒,右滑块需要调整位置
            tool.rightSpaceCons.constant = tool.cellWidth * CGFloat(6 - tool.dataList.count)
        }
        let totalDuration: CGFloat = CGFloat(CMTimeGetSeconds(tool.player!.totalDuration))
        tool.asecptNum = totalDuration / (tool.cellWidth * CGFloat(tool.dataList.count))
        tool.totalTime = totalDuration
        
        // 添加通知 进入前台
        NotificationCenter.default.addObserver(forName: NSNotification.Name.UIApplicationDidBecomeActive, object: nil, queue: OperationQueue.main, using: { (note) in
            
            tool.player?.resume()
        })
        NotificationCenter.default.addObserver(forName: NSNotification.Name.UIApplicationDidEnterBackground, object: nil, queue: OperationQueue.main, using: { (note) in
            
            tool.player?.pause()
        })
        
        tool.endTime = 15.0
        let timer = CADisplayLink.init(target: tool, selector: #selector(timeChangeAction))
        timer.add(to: RunLoop.main, forMode: .defaultRunLoopMode)
        tool.timer = timer
        
        return tool
    }
    
    override func awakeFromNib() {
        super.awakeFromNib()
        
        self.collectionView.delegate = self
        self.collectionView.dataSource = self
        
        self.layout.itemSize = CGSize.init(width: cellWidth, height: cellheight)
        self.layout.minimumLineSpacing = 0.0
        self.layout.minimumInteritemSpacing = 0.0
        
        self.collectionView.register(VideoImgCell.self, forCellWithReuseIdentifier: "VideoImgCell")
        
        // 设置选择时长的View间隔
        self.leftSpaceCons.constant = cellWidth - sliderWidth
        self.rightSpaceCons.constant = self.leftSpaceCons.constant
        
        // 自动偏移 一个单元格
        self.collectionView.contentInset = UIEdgeInsetsMake(0, CGFloat(Int(cellWidth)), 0, CGFloat(Int(cellWidth)))
        // 添加拖动手势
        self.leftView.addGestureRecognizer(self.leftPanGesture)
        self.rightView.addGestureRecognizer(self.rightPanGesture)
    }
    
    // 初始化播放器
    func setupPlayer(videoUrl: URL) {
        
        let option = PLPlayerOption.default()
        
        let player = PLPlayer.init(url: videoUrl, option: option)
        player?.delegate = self
        player?.playerView?.contentMode = .scaleAspectFit
        player?.isBackgroundPlayEnable = false
        self.playerView.addSubview(player!.playerView!)
        self.player = player
        self.player?.play()
    }
    
    //MARK: -点击事件
    @objc func panAction(pan: UIPanGestureRecognizer) {
       
        if pan.state == .began {
            
            self.stopPlay()
            
            if pan.view! == self.rightView {
                
                self.lastPlayTime = CGFloat(CMTimeGetSeconds(self.player!.currentTime))
            }
            
        } else if pan.state == .changed {
            
            var topFrame = self.topView.frame
            var bottomFrame = self.bottomView.frame
            
            let point = pan.location(in: pan.view!)
            if pan.view! == self.leftView {
                
                self.lastPlayTime = nil
                // 开始移动
                var newFrame = self.leftView.frame
                newFrame.origin.x += point.x + sliderWidth
                
                if (newFrame.origin.x >= self.leftSpaceCons.constant) && (newFrame.origin.x <= self.rightView.frame.origin.x - spaceMagin) {
                    
                    self.leftView.frame = newFrame
                    
                    let offsetX = newFrame.origin.x - cellWidth
                    let second = self.asecptNum * offsetX + self.leftScroTime
                    self.player?.seek(to: CMTime.init(seconds: Double(second), preferredTimescale: CMTimeScale(1 * NSEC_PER_SEC)))

                    self.leftTime = self.asecptNum * (offsetX + sliderWidth)
                    self.beginTime = self.leftScroTime + self.leftTime
                }
                topFrame.origin.x = self.leftView.frame.origin.x
                
            } else if pan.view! == self.rightView {
                
                // 开始移动
                var newFrame = self.rightView.frame
                newFrame.origin.x += point.x + sliderWidth
                // 15.0 + 10.0是右侧滑块空白区域的宽度
                if (newFrame.origin.x <= kWidth - self.rightSpaceCons.constant - 25.0) && (newFrame.origin.x >= self.leftView.frame.origin.x + spaceMagin) {
                    
                    self.rightView.frame = newFrame
                    
                    let offsetX = newFrame.origin.x - cellWidth
                    let second = self.asecptNum * offsetX + self.leftScroTime
                    self.player?.seek(to: CMTime.init(seconds: Double(second), preferredTimescale: CMTimeScale(1 * NSEC_PER_SEC)))

                    self.rightTime = self.asecptNum * (offsetX + 15.0)
                    self.endTime = self.rightTime + self.leftScroTime
                    if self.endTime >= self.totalTime {
                        
                        self.endTime = self.totalTime
                    }
                }
            }
            // 更新上下线条的frame
            topFrame.size.width = self.rightView.frame.origin.x - self.leftView.frame.origin.x + 15.0
            bottomFrame.origin.x = topFrame.origin.x
            bottomFrame.size.width = topFrame.size.width
            self.topView.frame = topFrame
            self.bottomView.frame = bottomFrame
            
        } else if pan.state == .ended {
            
            self.startPlay()
            // 计算左右滑块的距离
            let space = self.rightView.frame.origin.x - self.leftView.frame.origin.x + 15.0
            self.spaceDistance = space
        }
    }
    
    /// 定时器事件
    @objc func timeChangeAction() {
        
        if let player = self.player {
            
            let currentDuration: CGFloat = CGFloat(CMTimeGetSeconds(player.currentTime))
            if currentDuration >= self.endTime {
                
                player.seek(to: CMTime.init(seconds: Double(self.beginTime), preferredTimescale: CMTimeScale(1 * NSEC_PER_SEC)))
            }
            
            /*
            print("current:\(currentDuration),last:\(self.lastPlayTime ?? 0.0)")
            if self.lastPlayTime == nil {
                
                self.lastPlayTime = currentDuration
            }
            // 新增的数
            let addNum = currentDuration - (self.lastPlayTime ?? 0.0)
            let num = addNum / ((self.endTime - self.beginTime) / self.spaceDistance) + self.leftView.frame.origin.x
             */

            var newFrame = self.animationView.frame
            //newFrame.origin.x = num
            newFrame.origin.x += 0.34

            if newFrame.origin.x >= self.rightView.frame.origin.x + 20.0 {
                
                newFrame.origin.x = self.leftView.frame.origin.x + 9.0
            }
            self.animationView.frame = newFrame
        }
    }
    
    /// 开始播放执行的操作
    func startPlay() {

        DispatchQueue.main.async {

            // 防止重复调用
            if self.animationView.isHidden {
                
                // 重置动画View位置
                var newFrame = self.animationView.frame
                newFrame.origin.x = self.leftView.frame.origin.x + 9.0
                self.animationView.frame = newFrame
                self.animationView.isHidden = false
                
                self.player?.seek(to: CMTime.init(seconds: Double(self.beginTime), preferredTimescale: CMTimeScale(1 * NSEC_PER_SEC)))
                // 非静音
                self.player?.setVolume(0.3)
                self.player?.resume()
                // 添加定时器
                self.timer?.add(to: RunLoop.main, forMode: .defaultRunLoopMode)
            }
        }
    }
    
    /// 结束播放执行的操作
    func stopPlay() {
       
        DispatchQueue.main.async {
            
            // 防止重复调用
            if !self.animationView.isHidden {
                
                // 重置动画View位置
                self.animationView.isHidden = true
                // 静音
                self.player?.setVolume(0.0)
                // 移除定时器
                self.timer?.remove(from: RunLoop.main, forMode: .defaultRunLoopMode)
            }
        }
    }
    
    @IBAction func cancleAction() {
        
        DispatchQueue.main.async {
            
            NotificationCenter.default.removeObserver(self)
            self.player = nil
            AppDelegate.cureentRoot.popToRootViewController(animated: false)
            
            self.timer?.invalidate()
            self.timer = nil
            
            self.player?.delegate = nil
            self.player = nil
            NotificationCenter.default.removeObserver(self)
        }
    }
    
    @IBAction func finishAction() {
    
        self.videoClipMethod(fileUrl: self.videoUrl!, startTime: self.beginTime, endTime: self.endTime)
    }
    
    //MARK: -懒加载
    lazy var leftPanGesture: UIPanGestureRecognizer = {
        let pan = UIPanGestureRecognizer.init(target: self, action: #selector(panAction))
        return pan
    }()
    lazy var rightPanGesture: UIPanGestureRecognizer = {
        let pan = UIPanGestureRecognizer.init(target: self, action: #selector(panAction))
        return pan
    }()
}

extension VideoClipTool: PLPlayerDelegate {
    
    func player(_ player: PLPlayer, statusDidChange state: PLPlayerStatus) {
        
        // 结束播放
        if state == .statusCompleted {
            
            player.seek(to: CMTime.init(seconds: Double(self.beginTime), preferredTimescale: CMTimeScale(1 * NSEC_PER_SEC)))
        }
    }
}

extension VideoClipTool: UICollectionViewDataSource, UICollectionViewDelegateFlowLayout {

    func scrollViewDidScroll(_ scrollView: UIScrollView) {
        
        if let player = self.player {
            
            self.lastPlayTime = nil
            self.stopPlay()
            
            let offSetX = scrollView.contentOffset.x + CGFloat(Int(cellWidth))
            
            let second = self.asecptNum * offSetX + self.leftTime
            player.seek(to: CMTime.init(seconds: Double(second), preferredTimescale: CMTimeScale(1 * NSEC_PER_SEC)))
            self.leftScroTime = self.asecptNum * offSetX
            
            self.beginTime = self.leftScroTime + self.leftTime
            self.endTime = self.rightTime + self.leftScroTime
            if self.endTime >= self.totalTime {
                
                self.endTime = self.totalTime
            }
        }
    }
    
    func scrollViewDidEndScrollingAnimation(_ scrollView: UIScrollView) {
        
        self.startPlay()
    }
    
    func scrollViewDidEndDecelerating(_ scrollView: UIScrollView) {
        
        self.startPlay()
    }
    
    func scrollViewDidEndDragging(_ scrollView: UIScrollView, willDecelerate decelerate: Bool) {
        
        self.startPlay()
    }
    
    func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        
        return self.dataList.count
    }
    
    func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
        
        let cell = collectionView.dequeueReusableCell(withReuseIdentifier: "VideoImgCell", for: indexPath) as! VideoImgCell
        cell.imgV.image = self.dataList[indexPath.row]
        
        return cell
    }
    
    /// 裁剪视频
    func videoClipMethod(fileUrl: URL, startTime: CGFloat, endTime: CGFloat) {
        
        let asset = AVAsset.init(url: fileUrl)
        let exportSession : AVAssetExportSession! = AVAssetExportSession.init(asset: asset, presetName: AVAssetExportPresetPassthrough)
        
        // 移除之前保存的视频
        self.deleteTempFile(path: VideoSavePath)
        // 重新保存
        let fUrl: URL = URL.init(fileURLWithPath: VideoSavePath)
        exportSession.outputURL = fUrl
        exportSession.outputFileType = AVFileType.mp4
        
        let start = CMTimeMakeWithSeconds(Float64(startTime), 1)
        let duration = CMTimeMakeWithSeconds(Float64(endTime - startTime), 1)
        let range = CMTimeRange.init(start: start, duration: duration)
        exportSession.timeRange = range
        
        self.showLoading("视频处理中")
        exportSession.exportAsynchronously(completionHandler: {
            switch exportSession.status{
            case AVAssetExportSessionStatus.failed :
                
                self.showText("转换失败")
                print("Export failed " + (exportSession.error?.localizedDescription)!)
                break
                
            case AVAssetExportSessionStatus.cancelled:
                
                print("Export Canceled")
                self.showText("转换取消")
                break
                
            case AVAssetExportSessionStatus.completed:
                
                print("Export completed:\(fUrl)")
                self.showText("转换完成")
                self.stopPlay()
                weak var weakSelf = self
                // 保存到相册
                UISaveVideoAtPathToSavedPhotosAlbum((fUrl.relativePath), weakSelf, #selector(weakSelf?.videoDidFinishedSaved(videoPath:error:contextInfo:)), nil)
 
                self.cancleAction()
                self.dealVideoPathCallback?(VideoSavePath)
                
                break
                
            default:
                
                print("Export other status");
                break
            }
        })
    }
    // 是否保存完成
    @objc func videoDidFinishedSaved(videoPath:NSString, error:NSError, contextInfo:UnsafeMutableRawPointer?) {
        
        print(videoPath)
    }
    // 删除之前保存的文件
    func deleteTempFile(path: String) {
        let url = NSURL.fileURL(withPath: path)
        let filem = FileManager.default
        var error : NSError?
        if filem.fileExists(atPath: url.path) {
            
            do {
                try filem.removeItem(at: url)
                
            } catch let err as NSError {
                
                error = err
            }
            if (error != nil) {
                print("file remove error, " + (error?.localizedDescription)!)
            }
        } else {
            
            print("no file by that name")
        }
    }
    
    /// 获取视频截图 3秒一截图
    func getVideoThumbImg(videoUrl: URL) -> [UIImage?] {
        
        var arr: [UIImage?] = []
        let avAsset: AVAsset = AVAsset.init(url: videoUrl)
        //生成视频截图
        let generator = AVAssetImageGenerator(asset: avAsset)
        generator.appliesPreferredTrackTransform = true
        
        var seonds: Int = 1
        for _ in 0...Int(avAsset.duration.seconds) {
            
            if seonds <= Int(avAsset.duration.seconds) {
                
                let time = CMTimeMakeWithSeconds(Float64(seonds), 600)
                var actualTime:CMTime = CMTimeMake(0, 0)
                
                let imageRef: CGImage? = try? generator.copyCGImage(at: time, actualTime: &actualTime)
                
                var frameImg: UIImage?
                if let imageRef = imageRef {
                    
                    frameImg = UIImage.init(cgImage: imageRef)
                    arr.append(frameImg)
                }
                seonds += 3
                
            } else {
                
                break
            }
        }
        return arr
    }
}

class VideoImgCell: UICollectionViewCell {
    
    override init(frame: CGRect) {
        super.init(frame: frame)
        
        self.contentView.addSubview(self.imgV)
        
    }
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    lazy var imgV: UIImageView = {
        let img = UIImageView.init(frame: CGRect.init(x: 0.0, y: 0.0, width: self.contentView.frame.width, height: self.contentView.frame.size.height))
        img.contentMode = .scaleAspectFill
        img.clipsToBounds = true
        
        return img
    }()
    
}

